function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const loadEvents = async ({
  executeStatement,
  escapeId,
  escapeUnicode,
  tableName
}, {
  eventTypes,
  aggregateIds,
  startTime,
  finishTime,
  maxEventsByTimeframe = Number.POSITIVE_INFINITY
}, callback) => {
  const injectString = value => `${escapeUnicode(value)}`;

  const injectNumber = value => `${+value}`;

  const batchSize = 50;
  const queryConditions = [];

  if (eventTypes != null) {
    queryConditions.push(`${escapeId('type')} IN (${eventTypes.map(injectString)})`);
  }

  if (aggregateIds != null) {
    queryConditions.push(`${escapeId('aggregateId')} IN (${aggregateIds.map(injectString)})`);
  }

  if (startTime != null) {
    queryConditions.push(`${escapeId('timestamp')} > ${injectNumber(startTime)}`);
  }

  if (finishTime != null) {
    queryConditions.push(`${escapeId('timestamp')} < ${injectNumber(finishTime)}`);
  }

  const resultQueryCondition = queryConditions.length > 0 ? `WHERE ${queryConditions.join(' AND ')}` : '';
  let initialTimestamp = null;
  let countEvents = 0;

  loop: for (let skipCount = 0;; skipCount++) {
    const rows = await executeStatement(`SELECT * FROM ${escapeId(tableName)} ${resultQueryCondition}
      ORDER BY ${escapeId('eventId')} ASC
      LIMIT ${+(skipCount * batchSize)}, ${+batchSize}`);

    for (const event of rows) {
      if (initialTimestamp == null) {
        initialTimestamp = event.timestamp;
      }

      if (countEvents++ > maxEventsByTimeframe && event.timestamp !== initialTimestamp) {
        break loop;
      }

      await callback(_objectSpread({}, event, {
        payload: JSON.parse(event.payload)
      }));
    }

    if (rows.length < batchSize) {
      break;
    }
  }
};

export default loadEvents;
//# sourceMappingURL=load-events.js.map