"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

const coercer = (_ref) => {
  let {
    intValue,
    stringValue,
    bigIntValue,
    longValue
  } = _ref,
      rest = _objectWithoutProperties(_ref, ["intValue", "stringValue", "bigIntValue", "longValue"]);

  if (intValue != null) {
    return Number(intValue);
  } else if (bigIntValue != null) {
    return Number(bigIntValue);
  } else if (longValue != null) {
    return Number(longValue);
  } else if (stringValue != null) {
    return String(stringValue);
  } else {
    throw new Error(`Unknown type ${JSON.stringify(rest)}`);
  }
};

const executeStatement = async (pool, sql, transactionId = null) => {
  const result = await pool.rdsDataService.executeStatement(_objectSpread({}, transactionId != null ? {
    transactionId
  } : {}, {
    resourceArn: pool.config.dbClusterOrInstanceArn,
    secretArn: pool.config.awsSecretStoreArn,
    database: pool.config.databaseName,
    continueAfterTimeout: false,
    includeResultMetadata: true,
    sql
  })).promise();
  const {
    columnMetadata,
    records
  } = result;

  if (!Array.isArray(records) || columnMetadata == null) {
    return null;
  }

  const rows = [];

  for (const record of records) {
    const row = {};

    for (let i = 0; i < columnMetadata.length; i++) {
      row[columnMetadata[i].name] = coercer(record[i]);
    }

    rows.push(row);
  }

  return rows;
};

const beginTransaction = async pool => {
  const {
    transactionId
  } = await pool.rdsDataService.beginTransaction({
    resourceArn: pool.config.dbClusterOrInstanceArn,
    secretArn: pool.config.awsSecretStoreArn,
    database: pool.config.databaseName
  }).promise();
  return transactionId;
};

const commitTransaction = async (pool, transactionId) => {
  const {
    transactionStatus
  } = await pool.rdsDataService.commitTransaction({
    resourceArn: pool.config.dbClusterOrInstanceArn,
    secretArn: pool.config.awsSecretStoreArn,
    transactionId
  }).promise();
  return transactionStatus;
};

const rollbackTransaction = async (pool, transactionId) => {
  const {
    transactionStatus
  } = await pool.rdsDataService.rollbackTransaction({
    resourceArn: pool.config.dbClusterOrInstanceArn,
    secretArn: pool.config.awsSecretStoreArn,
    transactionId
  }).promise();
  return transactionStatus;
};

const connect = async (pool, {
  RDSDataService,
  escapeId,
  escapeUnicode,
  escape
}) => {
  const rdsDataService = new RDSDataService({
    region: pool.config.region
  });
  Object.assign(pool, {
    tableName: pool.config.tableName,
    rdsDataService,
    executeStatement: executeStatement.bind(null, pool),
    beginTransaction: beginTransaction.bind(null, pool),
    commitTransaction: commitTransaction.bind(null, pool),
    rollbackTransaction: rollbackTransaction.bind(null, pool),
    escapeUnicode,
    escapeId,
    escape,
    resourceOptions: pool.config.resourceOptions
  });
};

var _default = connect;
exports.default = _default;
//# sourceMappingURL=connect.js.map